#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .core import \
    IndentType, \
    StringBuilder, \
    BlockBuilder, \
    diff_lines, \
    diff_text, \
    interpolate_text, \
    timestamp, \
    write_text_file
